@@ -29,8 +29,8 @@
 /* ----------------------------------
  * version info
  * ---------------------------------- */
-static const char man_version[] = "10.3";
-static const char man_date[] = "February 2012";
+static const char man_version[] = "10.8";
+static const char man_date[] = "June 2012";
 
 /* ----------------------------------
  * definitions
