@@ -27,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsexec.c,v 1.12 2012/02/09 21:03:44 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsexec.c,v 1.15 2012/05/24 20:32:28 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
@@ -139,7 +139,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsfilter2(1), qsgeo(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsfilter2(1), qsgeo(1), qsgrep(1), qslog(1), qslogger(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -152,6 +152,10 @@ static void usage(char *cmd, int man) {
   }
 }
 
+/*
+ * Substitutes for $0-$9 within the matching string.
+ * See ap_pregsub().
+ */
 char *qs_pregsub(apr_pool_t *pool, const char *input,
 		 const char *source, size_t nmatch,
 		 regmatch_t pmatch[]) {
@@ -380,6 +384,7 @@ int main(int argc, const char * const argv[]) {
     nr++;
     if(pass) {
       printf("%s", line);
+      fflush(stdout);
     }
     if(clearpattern && (qs_regexec(clearpreg, line, MAX_REG_MATCH, regm) == 0)) {
       counter = 0;
