@@ -27,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsgrep.c,v 1.6 2012/02/09 21:01:38 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsgrep.c,v 1.8 2012/03/30 19:05:36 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
@@ -121,7 +121,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsexec(1), qsfilter2(1), qsgeo(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsexec(1), qsfilter2(1), qsgeo(1), qslog(1), qslogger(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -134,6 +134,10 @@ static void usage(char *cmd, int man) {
   }
 }
 
+/*
+ * Substitutes for $0-$9 within the matching string.
+ * See ap_pregsub().
+ */
 char *qs_pregsub(apr_pool_t *pool, const char *input,
 		 const char *source, size_t nmatch,
 		 regmatch_t pmatch[]) {
