@@ -1,3 +1,6 @@
+/* -*-mode: c; indent-tabs-mode: nil; c-basic-offset: 2; -*-
+ */
+
 /**
  * Utilities for the quality of service module mod_qos.
  *
@@ -25,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.40 2012/02/14 19:32:30 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.46 2012/05/15 18:51:27 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -89,7 +92,8 @@ typedef struct stat_rec_st {
   long line_count;
   long long i_byte_count;
   long long byte_count;
-  long duration_count;
+  long long duration_count;
+  long long duration_count_ms;
   long duration_0;
   long duration_1;
   long duration_2;
@@ -98,13 +102,13 @@ typedef struct stat_rec_st {
   long duration_5;
   long duration_6;
   long connections;
-  
+
   long status_1;
   long status_2;
   long status_3;
   long status_4;
   long status_5;
-  
+
   long qos_v;
   long qos_s;
   long qos_d;
@@ -137,6 +141,7 @@ static int   m_offline = 0;
 static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
 static int   m_mem = 0;
+static int   m_avms = 0;
 static apr_table_t *m_client_entries = NULL;
 static int   m_max_client_entries = 0;
 static int   m_offline_count = 0;
@@ -144,6 +149,10 @@ static int   m_offline_count = 0;
 static long  m_lines = 0;
 static int   m_verbose = 0;
 
+/**
+ * Helper to print an error message when terminating
+ * the programm due to an unexpected error.
+ */
 static void qerror(const char *fmt,...) {
   char buf[MAX_LINE];
   va_list args;
@@ -167,7 +176,7 @@ static char *skipElement(const char* line) {
     p++;
     // read while we found an '" '" which is not escaped
     while(p[0] != 0 &&
-	  !(p[0] == delim && p[-1] != '\\' && (p[1] == '\0' || p[1] == ' '))) {
+          !(p[0] == delim && p[-1] != '\\' && (p[1] == '\0' || p[1] == ' '))) {
       p++;
     }
     p++;
@@ -177,20 +186,20 @@ static char *skipElement(const char* line) {
       // offline mode: check for <name>='<value>' entry
       eq = strstr(p, "='");
       if(eq && (eq - p) < 10) {
-	// near hit
-	p = &eq[3];
-	while(p[0] != '\'' && p[0] != 0 && p[-1] != '\\') {
-	  p++;
-	}
-	p++;
+        // near hit
+        p = &eq[3];
+        while(p[0] != '\'' && p[0] != 0 && p[-1] != '\\') {
+          p++;
+        }
+        p++;
       } else {
-	// something else...
-	eq=NULL;
+        // something else...
+        eq=NULL;
       }
     }
     if(!eq) {
       while(p[0] != ' ' && p[0] != 0) {
-	p++;
+        p++;
       }
     }
   }
@@ -200,6 +209,9 @@ static char *skipElement(const char* line) {
   return p;
 }
 
+/**
+ * Strip a number.
+ */
 static void stripNum(char **p) {
   char *s = *p;
   int len;
@@ -214,8 +226,11 @@ static void stripNum(char **p) {
   *p = s;
 }
 
-/*
- * get and cut an element
+/**
+ * Get and cut an element.
+ *
+ * @param line Line to parse for the next element.
+ * @return Pointer to the next element.
  */
 static char *cutNext(char **line) {
   char *c = *line;
@@ -239,6 +254,14 @@ static char *cutNext(char **line) {
   return c;
 }
 
+/**
+ * Calculates the free system memory. Experimental code.
+ * Tested on Solaris (calling vmstat) and Linux (reading
+ * from /proc/meminfo).
+ *
+ * @param buf Buffer to write result to
+ * @sz Max. length of the buffer
+ */
 static void getFreeMem(char *buf, int sz) {
   FILE *f = fopen("/proc/meminfo", "r");
   int mem = 0;
@@ -247,22 +270,22 @@ static void getFreeMem(char *buf, int sz) {
     char line[MAX_LINE];
     while(!qs_getLinef(line, sizeof(line), f)) {
       if(strncmp(line, "MemFree: ", 9) == 0) {
-	char *c = &line[9];
-	char *e;
-	while(c[0] && ((c[0] == ' ') || (c[0] == '\t'))) c++;
-	e = c;
-	while(e[0] && (e[0] != ' ')) e++;
-	e[0] = '\0';
-	mem = mem + atoi(c);
+        char *c = &line[9];
+        char *e;
+        while(c[0] && ((c[0] == ' ') || (c[0] == '\t'))) c++;
+        e = c;
+        while(e[0] && (e[0] != ' ')) e++;
+        e[0] = '\0';
+        mem = mem + atoi(c);
       }
       if(strncmp(line, "Cached: ", 8) == 0) {
-	char *c = &line[8];
-	char *e;
-	while(c[0] && ((c[0] == ' ') || (c[0] == '\t'))) c++;
-	e = c;
-	while(e[0] && (e[0] != ' ')) e++;
-	e[0] = '\0';
-	mem = mem + atoi(c);
+        char *c = &line[8];
+        char *e;
+        while(c[0] && ((c[0] == ' ') || (c[0] == '\t'))) c++;
+        e = c;
+        while(e[0] && (e[0] != ' ')) e++;
+        e[0] = '\0';
+        mem = mem + atoi(c);
       }
     }
     fclose(f);
@@ -272,20 +295,19 @@ static void getFreeMem(char *buf, int sz) {
     char vmstat[] = "/usr/bin/vmstat";
     struct stat attr;
     if(stat(vmstat, &attr) == 0) {
-      int rc;
       char command[1024];
       char outfile[1024];
       snprintf(outfile, sizeof(outfile), "/tmp/qslog.%d", getpid());
       snprintf(command, sizeof(command), "%s 1 2 1>%s", vmstat, outfile);
-      rc = system(command);
+      system(command);
       f = fopen(outfile, "r");
       if(f) {
         char line[MAX_LINE];
         int i = 1;
         while(!qs_getLinef(line, sizeof(line), f)) {
           if(i == 4) {
-	    // free memory only (ignores cache on linux)
-	    int j = 0;
+            // free memory only (ignores cache on linux)
+            int j = 0;
             char *p = line;
             while(p && j < 4) {
               p++;
@@ -298,7 +320,7 @@ static void getFreeMem(char *buf, int sz) {
               e = strchr(p, ' ');
               if(e) {
                 e[0] = '\0';
-		snprintf(buf, sz, "%s", p);
+                snprintf(buf, sz, "%s", p);
               }
             }
             break;
@@ -306,7 +328,7 @@ static void getFreeMem(char *buf, int sz) {
           i++;
         }
         fclose(f);
-	unlink(outfile);
+        unlink(outfile);
       }
     }
   }
@@ -317,21 +339,42 @@ static void getFreeMem(char *buf, int sz) {
 #define NBS "b/s"
 #define NBIS "ib/s"
 #define NAV "av"
+#define NAVMS "avms"
 
-static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int offline, int main,
-			   double *av, const char *mem) {
+/**
+ * Writes the statistic entry stat_rec to the file.
+ *
+ * @param f File to write to
+ * @param timeStr Time string (prefix)
+ * @param stat_rec Data to write
+ * @offline Offline mode (less data, e.g. no load)
+ * @param main Indicates if it is the main log or a sub entry for the detailed log
+ * @param av Load
+ * @param mem Free memory
+ */
+static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
+                           int offline, int main,
+                           double *av, const char *mem) {
   char bis[256];
   char esco[256];
   char ip[256];
   char usr[256];
+  char avms[256];
   bis[0] = '\0';
   esco[0] = '\0';
+  avms[0] = '\0';
   if(stat_rec->i_byte_count != -1) {
     sprintf(bis, NBIS";%lld;", stat_rec->i_byte_count/LOG_INTERVAL);
   }
   if(main && stat_rec->connections != -1) {
     sprintf(esco, "esco;%ld;", stat_rec->connections);
   }
+  if(m_avms) {
+    sprintf(avms, NAVMS";%lld;",
+            stat_rec->duration_count_ms/(stat_rec->line_count == 0 ? 1 : stat_rec->line_count));
+    // improve accuracy (rounding errors):
+    stat_rec->duration_count = stat_rec->duration_count_ms / 1000;
+  }
   if(main) {
     sprintf(ip, "ip;%ld;", qs_countEvent(&m_ip_list));
     sprintf(usr, "usr;%ld;", qs_countEvent(&m_user_list));
@@ -341,18 +384,19 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
   }
 
   fprintf(f, "%s;"
-	  "%s"
+          "%s"
           NRS";%ld;"
-	  "req;%ld;"
+          "req;%ld;"
           NBS";%lld;"
-	  "%s"
-	  "%s"
-	  "1xx;%ld;"
-	  "2xx;%ld;"
-	  "3xx;%ld;"
-	  "4xx;%ld;"
-	  "5xx;%ld;"
-          NAV";%ld;"
+          "%s"
+          "%s"
+          "1xx;%ld;"
+          "2xx;%ld;"
+          "3xx;%ld;"
+          "4xx;%ld;"
+          "5xx;%ld;"
+          "%s"
+          NAV";%lld;"
           "<1s;%ld;"
           "1s;%ld;"
           "2s;%ld;"
@@ -360,28 +404,29 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
           "4s;%ld;"
           "5s;%ld;"
           ">5s;%ld;"
-	  "%s"
-	  "%s"
-	  "qV;%ld;"
-	  "qS;%ld;"
-	  "qD;%ld;"
-	  "qK;%ld;"
-	  "qT;%ld;"
-	  "qL;%ld;"
-	  "qs;%ld;"
-	  ,
-	  timeStr,
-	  main ? "" : stat_rec->id,
+          "%s"
+          "%s"
+          "qV;%ld;"
+          "qS;%ld;"
+          "qD;%ld;"
+          "qK;%ld;"
+          "qT;%ld;"
+          "qL;%ld;"
+          "qs;%ld;"
+          ,
+          timeStr,
+          main ? "" : stat_rec->id,
           stat_rec->line_count/LOG_INTERVAL,
-	  stat_rec->line_count,
+          stat_rec->line_count,
           stat_rec->byte_count/LOG_INTERVAL,
-	  bis,
-	  esco,
-	  stat_rec->status_1,
-	  stat_rec->status_2,
-	  stat_rec->status_3,
-	  stat_rec->status_4,
-	  stat_rec->status_5,
+          bis,
+          esco,
+          stat_rec->status_1,
+          stat_rec->status_2,
+          stat_rec->status_3,
+          stat_rec->status_4,
+          stat_rec->status_5,
+          avms,
           stat_rec->duration_count/(stat_rec->line_count == 0 ? 1 : stat_rec->line_count),
           stat_rec->duration_0,
           stat_rec->duration_1,
@@ -390,15 +435,15 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
           stat_rec->duration_4,
           stat_rec->duration_5,
           stat_rec->duration_6,
-	  ip,
-	  usr,
-	  stat_rec->qos_v,
-	  stat_rec->qos_s,
-	  stat_rec->qos_d,
-	  stat_rec->qos_k,
-	  stat_rec->qos_t,
-	  stat_rec->qos_l,
-	  stat_rec->qos_ser
+          ip,
+          usr,
+          stat_rec->qos_v,
+          stat_rec->qos_s,
+          stat_rec->qos_d,
+          stat_rec->qos_k,
+          stat_rec->qos_t,
+          stat_rec->qos_l,
+          stat_rec->qos_ser
           );
   stat_rec->line_count = 0;
   stat_rec->byte_count = 0;
@@ -414,6 +459,7 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
   stat_rec->status_4 = 0;
   stat_rec->status_5 = 0;
   stat_rec->duration_count = 0;
+  stat_rec->duration_count_ms = 0;
   stat_rec->duration_0 = 0;
   stat_rec->duration_1 = 0;
   stat_rec->duration_2 = 0;
@@ -431,7 +477,7 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
   if(main) {
     if(!offline) {
       fprintf(f, "sl;%.2f;m;%s",
-	      av[0], mem[0] ? mem : "-");
+              av[0], mem[0] ? mem : "-");
     } else {
       fprintf(f, "sl;-;m;-");
       m_offline_data = 0;
@@ -440,6 +486,13 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int off
   fprintf(f, "\n");
 }
 
+/**
+ * Creates and init new status rec
+ *
+ * @param id Identification of the id
+ * @param pattern Pattern to match the log data line
+ * @return
+ */
 static stat_rec_t *createRec(const char *id, const char *pattern) {
   stat_rec_t *rec = calloc(sizeof(stat_rec_t), 1);
   rec->id = calloc(strlen(id)+2, 1);
@@ -455,6 +508,7 @@ static stat_rec_t *createRec(const char *id, const char *pattern) {
   rec->i_byte_count = -1;
   rec->byte_count = 0;
   rec->duration_count = 0;
+  rec->duration_count_ms = 0;
   rec->duration_0 = 0;
   rec->duration_1 = 0;
   rec->duration_2 = 0;
@@ -480,6 +534,12 @@ static stat_rec_t *createRec(const char *id, const char *pattern) {
   return rec;
 }
 
+/**
+ * Retrieves the best matching record (longest match(
+ * 
+ * @param Parameter to match, e.g. URL
+ * @return Matching entry (NULL if no match)
+ */
 static stat_rec_t *getRec(const char *value) {
   regmatch_t ma[1];
   int len = 0;
@@ -489,9 +549,9 @@ static stat_rec_t *getRec(const char *value) {
     if(regexec(&r->preg, value, 1, ma, 0) == 0) {
       int l = ma[0].rm_eo - ma[0].rm_so + 1;
       if(l > len) {
-	// longest match
-	len = l;
-	rec = r;
+        // longest match
+        len = l;
+        rec = r;
       }
     }
     r = r->next;
@@ -499,9 +559,11 @@ static stat_rec_t *getRec(const char *value) {
   return rec;
 }
 
-/*
+/**
  * writes all stat data to the out file
- * an resets all counters
+ * an resets all counters.
+ *
+ * @param timeStr
  */
 static void printAndResetStat(char *timeStr) {
   stat_rec_t *r = m_stat_sub;
@@ -530,12 +592,12 @@ static void printAndResetStat(char *timeStr) {
   }
 }
 
+/**
+ * Updates the per client record
+ */
 static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
-			 char *BI, char *B, char *R, char *I, char *U, char *Q, char *k, char *C,
-			 long tme) {
-  long long start;
-  long long end;
-  struct timeval tv;
+                         char *BI, char *B, char *R, char *I, char *U, char *Q,
+                         char *k, char *C, long tme) {
   client_rec_t *client_rec;
   const char *id = I; // ip
   if(id == NULL) {
@@ -550,8 +612,8 @@ static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
     if(apr_table_elts(m_client_entries)->nelts >= MAX_CLIENT_ENTRIES) {
       // limitation: speed (table to big) and memory
       if(!m_max_client_entries) {
-	fprintf(stderr, "\nreached max client enries (%d)\n", MAX_CLIENT_ENTRIES);
-	m_max_client_entries = 1;
+        fprintf(stderr, "\nreached max client enries (%d)\n", MAX_CLIENT_ENTRIES);
+        m_max_client_entries = 1;
       }
       return;
     }
@@ -614,9 +676,12 @@ static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
   return;
 }
 
+/**
+ * Updates standard record
+ */
 static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
-		      char *BI, char *B, char *R, char *I, char *U, char *Q, char *k, char *C,
-		      long tme) {
+                      char *BI, char *B, char *R, char *I, char *U, char *Q,
+                      char *k, char *C, long tme, long tmems) {
   if(Q != NULL) {
     if(strchr(Q, 'V') != NULL) {
       rec->qos_v++;
@@ -679,6 +744,7 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
   if(T != NULL || t != NULL || D != NULL) {
     /* response duration */
     rec->duration_count += tme;
+    rec->duration_count_ms += tmems;
     if(tme < 1) {
       rec->duration_0++;
     } else if(tme == 1) {
@@ -728,6 +794,7 @@ static void updateStat(const char *cstr, char *line) {
   const char *c = cstr;
   char *l = line;
   long tme;
+  long tmems;
   if(!line[0]) return;
   while(c[0]) {
     /* process known types */
@@ -815,20 +882,23 @@ static void updateStat(const char *cstr, char *line) {
   tme = 0;
   if(T != NULL || t != NULL || D != NULL) {
     /* response duration */
+    tmems = 0;
     if(T) {
       stripNum(&T);
       tme = atol(T);
     } else if(t) {
       stripNum(&t);
-      tme = atol(t) / 1000;
+      tmems= atol(t);
+      tme = tmems / 1000;
     } else if(D) {
       stripNum(&D);
-      tme = atol(D) / 1000000;
+      tmems = atol(D) / 1000;
+      tme = tmems / 1000;
     }
   }
-  updateRec(m_stat_rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+  updateRec(m_stat_rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme, tmems);
   if(rec) {
-    updateRec(rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+    updateRec(rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme, tmems);
   }
   if(m_offline_count) {
     updateClient(NULL, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
@@ -838,14 +908,14 @@ static void updateStat(const char *cstr, char *line) {
   if(m_verbose && (m_offline || m_offline_count)) {
     m_lines++;
     printf("[%ld] I=[%s] U=[%s] B=[%s] i=[%s] S=[%s] T=[%ld] Q=[%s]\n", m_lines,
-	   I == NULL ? "(null)" : I,
-	   U == NULL ? "(null)" : U,
-	   B == NULL ? "(null)" : B,
-	   BI == NULL ? "(null)" : BI,
-	   S == NULL ? "(null)" : S,
-	   tme,
-	   Q == NULL ? "(null)" : Q
-	   );
+           I == NULL ? "(null)" : I,
+           U == NULL ? "(null)" : U,
+           B == NULL ? "(null)" : B,
+           BI == NULL ? "(null)" : BI,
+           S == NULL ? "(null)" : S,
+           tme,
+           Q == NULL ? "(null)" : Q
+           );
   }
   line[0] = '\0';
 }
@@ -892,19 +962,19 @@ static time_t getMinutes(char *line) {
       minutes = minutes + (atoi(&buf[strlen(buf)-2]) * 60);
       /* store date information */
       {
-	char *year;
-	char *month;
-	char *day;
-	/* cut hours */
-	buf[strlen(buf)-3] = '\0';
-	day = &buf[strlen(buf)-2];
-	/* cut day */
-	buf[strlen(buf)-3] = '\0';
-	month = &buf[strlen(buf)-2];
-	/* cut month */
-	buf[strlen(buf)-3] = '\0';
-	year = buf;
-	snprintf(m_date_str, sizeof(m_date_str), "%s.%s.%s", day, month, year);
+        char *year;
+        char *month;
+        char *day;
+        /* cut hours */
+        buf[strlen(buf)-3] = '\0';
+        day = &buf[strlen(buf)-2];
+        /* cut day */
+        buf[strlen(buf)-3] = '\0';
+        month = &buf[strlen(buf)-2];
+        /* cut month */
+        buf[strlen(buf)-3] = '\0';
+        year = buf;
+        snprintf(m_date_str, sizeof(m_date_str), "%s.%s.%s", day, month, year);
       }
       return minutes;
     } else {
@@ -954,7 +1024,16 @@ static time_t getMinutes(char *line) {
  */
 static void readStdin(const char *cstr) {
   char line[MAX_LINE];
-  while(qs_getLine(line, sizeof(line))) {
+  int line_len;
+  while(fgets(line, sizeof(line), stdin) != NULL) {
+    line_len = strlen(line) - 1;
+    while(line_len > 0) { // cut tailing CR/LF
+      if(line[line_len] >= ' ') {
+        break;
+      }
+      line[line_len] = '\0';
+      line_len--;
+    }
     updateStat(cstr, line);
   }
 }
@@ -970,8 +1049,18 @@ static void readStdinOffline(const char *cstr) {
   char line[MAX_LINE];
   char buf[32];
   time_t unitTime = 0;
-  while(qs_getLine(line, sizeof(line))) {
-    time_t l_time = getMinutes(line);
+  int line_len;
+  while(fgets(line, sizeof(line), stdin) != NULL) {
+    time_t l_time;
+    line_len = strlen(line) - 1;
+    while(line_len > 0) { // cut tailing CR/LF
+      if(line[line_len] >= ' ') {
+        break;
+      }
+      line[line_len] = '\0';
+      line_len--;
+    }
+    l_time = getMinutes(line);
     m_offline_data = 1;
     if(unitTime == 0) {
       unitTime = l_time;
@@ -987,18 +1076,18 @@ static void readStdinOffline(const char *cstr) {
       unitTime = 0;
     } else {
       if(l_time > unitTime) {
-      	if(!m_verbose) {
-	  fprintf(stdout, ".");
-	  fflush(stdout);
-	}
+        if(!m_verbose) {
+          fprintf(stdout, ".");
+          fflush(stdout);
+        }
       }
       while(l_time > unitTime) {
-	snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
-	if(m_offline) {
-	  printAndResetStat(buf);
-	}
-	unitTime++;
-	qs_setTime(unitTime * 60);;
+        snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
+        if(m_offline) {
+          printAndResetStat(buf);
+        }
+        unitTime++;
+        qs_setTime(unitTime * 60);;
       }
       updateStat(cstr, line);
     }
@@ -1031,34 +1120,37 @@ static void *loggerThread(void *argv) {
     if(m_rotate) {
       strftime(buf, sizeof(buf), "%H:%M", ptr);
       if(strcmp(buf, "23:59") == 0) {
-	char arch[MAX_LINE];
-	char arch2[MAX_LINE];
-	strftime(buf, sizeof(buf), "%Y%m%d%H%M%S", ptr);
-	snprintf(arch, sizeof(arch), "%s.%s", m_file_name, buf);
-	snprintf(arch2, sizeof(arch), "%s.%s", m_file_name2, buf);
-	if(fclose(m_f) != 0) {
-	  qerror("failed to close file '%s': %s", m_file_name, strerror(errno));
-	}
-	if(rename(m_file_name, arch) != 0) {
-	  qerror("failed to move file '%s': %s", arch, strerror(errno));
-	}
-	m_f = fopen(m_file_name, "a+"); 
-	if(m_f2) {
-	  fclose(m_f2);
-	  rename(m_file_name2, arch2);
-	  m_f2 = fopen(m_file_name2, "a+"); 
-	}
+        char arch[MAX_LINE];
+        char arch2[MAX_LINE];
+        strftime(buf, sizeof(buf), "%Y%m%d%H%M%S", ptr);
+        snprintf(arch, sizeof(arch), "%s.%s", m_file_name, buf);
+        snprintf(arch2, sizeof(arch), "%s.%s", m_file_name2, buf);
+        if(fclose(m_f) != 0) {
+          qerror("failed to close file '%s': %s", m_file_name, strerror(errno));
+        }
+        if(rename(m_file_name, arch) != 0) {
+          qerror("failed to move file '%s': %s", arch, strerror(errno));
+        }
+        m_f = fopen(m_file_name, "a+");
+        if(m_f2) {
+          fclose(m_f2);
+          rename(m_file_name2, arch2);
+          m_f2 = fopen(m_file_name2, "a+");
+        }
       }
     }
   }
   return NULL;
 }
 
+/**
+ * usage text
+ */
 static void usage(const char *cmd, int man) {
   if(man) {
     //.TH [name of program] [section number] [center footer] [left footer] [center header]
     printf(".TH %s 1 \"%s\" \"mod_qos utilities %s\" \"%s man page\"\n", qs_CMD(cmd), man_date,
-	   man_version, cmd);
+           man_version, cmd);
   }
   printf("\n");
   if(man) {
@@ -1085,6 +1177,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "  - bytes received from the client per second ("NBIS")\n");
   qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
   qs_man_println(man, "  - average response duration ("NAV")\n");
+  qs_man_println(man, "  - average response duration in milliseconds ("NAVMS")\n");
   qs_man_println(man, "  - distribution of response durations within the last minute\n");
   qs_man_print(man, "    (<1s,1s,2s,3s,4s,5s,>5)\n");
   if(man) printf("\n");
@@ -1117,8 +1210,8 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     B defines the transferred bytes (%%b or %%O)\n");
   qs_man_println(man, "     i defines the received bytes (%%I)\n");
   qs_man_println(man, "     T defines the request duration (%%T)\n");
-  qs_man_println(man, "     t defines the request duration in milliseconds (optionally used instead of T)\n");
-  qs_man_println(man, "     D defines the request duration in microseconds (optionally used instead of T) (%%D)\n");
+  qs_man_println(man, "     t defines the request duration in milliseconds (may be used instead of T)\n");
+  qs_man_println(man, "     D defines the request duration in microseconds (may be used instead of T) (%%D)\n");
   qs_man_println(man, "     k defines the number of keepalive requests on the connection (%%k)\n");
   qs_man_println(man, "     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
   qs_man_println(man, "     Q defines the mod_qos_ev event message (%%{mod_qos_ev}e)\n");
@@ -1191,7 +1284,7 @@ static void usage(const char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsexec(1), qsfilter2(1), qsgeo(1), qsgrep(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsexec(1), qsfilter2(1), qsgeo(1), qsgrep(1), qslogger(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -1204,6 +1297,12 @@ static void usage(const char *cmd, int man) {
   }
 }
 
+/**
+ * Loads the rule files. Each rule (pattern) is prefixed by an id.
+ *
+ * @param confFile Path to the rule file to load
+ * @return
+ */
 static stat_rec_t *loadRule(const char *confFile) {
   char line[MAX_LINE];
   FILE *file = fopen(confFile, "r"); 
@@ -1221,16 +1320,16 @@ static stat_rec_t *loadRule(const char *confFile) {
       p[0] = '\0';
       p++;
       if(m_verbose) {
-	printf("load rule %s: %s\n", id, p);
+        printf("load rule %s: %s\n", id, p);
       }
       next = createRec(id, p);
       if(rec == NULL) {
-	rec = next;
+        rec = next;
       }
       if(prev) {
-	prev->next = next;
+        prev->next = next;
       } else {
-	rec->next = next;
+        rec->next = next;
       }
       prev = next;
     }
@@ -1261,27 +1360,31 @@ int main(int argc, const char *const argv[]) {
   while(argc >= 1) {
     if(strcmp(*argv,"-f") == 0) { /* this is the format string */
       if (--argc >= 1) {
-	config = *(++argv);
-	if(strchr(config, 'i')) {
-	  // enable ib/s
-	  m_stat_rec->i_byte_count = 0;
-	}
-	if(strchr(config, 'k')) {
-	  // enable esco
-	  m_stat_rec->connections = 0;
-	}
+        config = *(++argv);
+        if(strchr(config, 'i')) {
+          // enable ib/s
+          m_stat_rec->i_byte_count = 0;
+        }
+        if(strchr(config, 'k')) {
+          // enable esco
+          m_stat_rec->connections = 0;
+        }
+        if(strchr(config, 'D') || strchr(config, 't')) {
+          // enable average duration in ms
+          m_avms = 1;
+        }
       }
     } else if(strcmp(*argv,"-o") == 0) { /* this is the out file */
       if (--argc >= 1) {
-	file = *(++argv);
+        file = *(++argv);
       }
     } else if(strcmp(*argv,"-u") == 0) { /* switch user id */
       if (--argc >= 1) {
-	username = *(++argv);
+        username = *(++argv);
       }
     } else if(strcmp(*argv,"-c") == 0) { /* custom patterns (e.g. url pattern list, format: <id>':'<pattern>) */
       if (--argc >= 1) {
-	confFile = *(++argv);
+        confFile = *(++argv);
       }
     } else if(strcmp(*argv,"-p") == 0) { /* activate offline analysis */
       m_offline = 1;
@@ -1314,19 +1417,25 @@ int main(int argc, const char *const argv[]) {
   if(m_offline || m_offline_count) {
     /* init time pattern regex, std apache access log */
     regcomp(&m_trx, 
-	    "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
-	    REG_EXTENDED);
+            "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
+            REG_EXTENDED);
     /* other time patterns: "yyyy mm dd hh:mm:ss,mmm" or "yyyy mm dd hh:mm:ss.mmm" */
     regcomp(&m_trx2, 
-	    "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
-	    REG_EXTENDED);
+            "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
+            REG_EXTENDED);
   }
 
+  /*
+   * offline count mode creates statitis
+   * on a per client basis (e.g. per source
+   * ip or user id using the user tracking
+   * feature of mod_qos)
+   */
   if(m_offline_count) {
     int i;
     apr_table_entry_t *entry;
     apr_pool_t *pool;
-    int rc = nice(10);
+    nice(10);
     if(config == NULL) usage(cmd, 0);
     apr_app_initialize(&argc, &argv, NULL);
     apr_pool_create(&pool, NULL);
@@ -1337,32 +1446,32 @@ int main(int argc, const char *const argv[]) {
     m_f = stdout;
     if(file) {
       m_f = fopen(file, "a+");
-      if(m_f == NULL) {
-	m_f == stdout;
+      if(!m_f) {
+        m_f == stdout;
       }
     }
     for(i = 0; i < apr_table_elts(m_client_entries)->nelts; i++) {
       client_rec_t *client_rec = (client_rec_t *)entry[i].val;
       fprintf(m_f, "%s;req;%ld;errors;%ld;"
-	      "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
-	      "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;"
-	      "\n",
-	      entry[i].key,
-	      client_rec->request_count,
-	      client_rec->error_count,
-	      client_rec->status_1,
-	      client_rec->status_2,
-	      client_rec->status_3,
-	      client_rec->status_4,
-	      client_rec->status_5,
-	      client_rec->duration / client_rec->request_count,
-	      client_rec->duration_0,
-	      client_rec->duration_1,
-	      client_rec->duration_2,
-	      client_rec->duration_3,
-	      client_rec->duration_4,
-	      client_rec->duration_5,
-	      client_rec->duration_6);
+              "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
+              "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;"
+              "\n",
+              entry[i].key,
+              client_rec->request_count,
+              client_rec->error_count,
+              client_rec->status_1,
+              client_rec->status_2,
+              client_rec->status_3,
+              client_rec->status_4,
+              client_rec->status_5,
+              client_rec->duration / client_rec->request_count,
+              client_rec->duration_0,
+              client_rec->duration_1,
+              client_rec->duration_2,
+              client_rec->duration_3,
+              client_rec->duration_4,
+              client_rec->duration_5,
+              client_rec->duration_6);
     }
     if(file && m_f != stdout) {
       fclose(m_f);
@@ -1419,8 +1528,13 @@ int main(int argc, const char *const argv[]) {
     }
   }
 
+  /*
+   * Offline mode reads an existing file
+   * adjusting a virtual clock based on
+   * the date string match of the log
+   * enties. */
   if(m_offline) {
-    int rc = nice(10);
+    nice(10);
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
     readStdinOffline(config);
@@ -1428,6 +1542,10 @@ int main(int argc, const char *const argv[]) {
       fprintf(stdout, "\n");
     }
   } else {
+    /* standard mode reads data from
+     * stdin and uses a separate thread
+     * to write the data every minute. 
+     */
     pthread_create(&tid, tha, loggerThread, NULL);
     readStdin(config);
   }
